/*
 * main.c - for periodicstats everything starts here.
 *
 * Copyright (C) 2022 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Shubham Sharma <shubham.sharma@iopsys.eu>
 *
 * See LICENSE file for license related information.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <libubus.h>

#include "periodicstats.h"

#define DM_OBJECT_NAME "bbfdm"
// Global periodic stats structure
static LIST_HEAD(ss_list);

/**
 *  This function prints the usage of the daemon
 *  @param prog input pointer to char, for program name
 */
static void pstat_usage(char *prog)
{
	fprintf(stderr, "Usage: %s [options]\n", prog);
	fprintf(stderr, "\n");
	fprintf(stderr, "options:\n");
	fprintf(stderr, "    -s <socket path>   ubus socket\n");
	fprintf(stderr, "    -h <help>       To print this help menu\n");
	fprintf(stderr, "\n");
}

static bool g_usp_object_available = false;

static void lookup_event_cb(struct ubus_context *ctx, struct ubus_event_handler *ev,
			  const char *type, struct blob_attr *msg)
{
	const struct blobmsg_policy policy = {
		"path", BLOBMSG_TYPE_STRING
	};
	struct blob_attr *attr;
	const char *path;

	if (strcmp(type, "ubus.object.add") != 0)
		return;

	blobmsg_parse(&policy, 1, &attr, blob_data(msg), blob_len(msg));
	if (!attr)
		return;

	path = blobmsg_data(attr);
	if (strcmp(path, DM_OBJECT_NAME) == 0) {
		g_usp_object_available = true;
		uloop_end();
	}
}

static void lookup_timeout_cb(struct uloop_timeout *timeout)
{
	uloop_end();
}

int wait_for_dm_object(void)
{
#define USP_RAW_WAIT_TIMEOUT 60

	struct ubus_context *uctx;
	int ret;
	uint32_t ubus_id;
	struct ubus_event_handler add_event;
	struct uloop_timeout u_timeout;

	g_usp_object_available = false;
	uctx = ubus_connect(NULL);
	if (uctx == NULL) {
		fprintf(stderr, "Can't create ubus context");
		return -1;
	}

	uloop_init();
	ubus_add_uloop(uctx);

	// register for add event
	memset(&add_event, 0, sizeof(struct ubus_event_handler));
	add_event.cb = lookup_event_cb;
	ubus_register_event_handler(uctx, &add_event, "ubus.object.add");

	// check if object already present
	ret = ubus_lookup_id(uctx, DM_OBJECT_NAME, &ubus_id);
	if (ret == 0) {
		g_usp_object_available = true;
		goto end;
	}

	// Set timeout to expire lookup
	memset(&u_timeout, 0, sizeof(struct uloop_timeout));
	u_timeout.cb = lookup_timeout_cb;
	uloop_timeout_set(&u_timeout, 60 * 1000);

	uloop_run();
	uloop_done();

end:
	ubus_free(uctx);

	if (g_usp_object_available == false) {
		fprintf(stderr, "### %s object not found", DM_OBJECT_NAME);
		return -1;
	}

	return 0;
}

/**
 *  Main function for periodicstats, everything starts here
 *  @param argc input number of input arguments
 *  @param argv input double pointer array of optional command line arguments
 *  retrun integer value 0 on success and -1 on failure
 */
int main(int argc, char **argv)
{
	struct ubus_context *ctx = NULL;
	const char *ubus_socket = NULL;
	int ch, ret;

	while ((ch = getopt(argc, argv, "hs:")) != -1) {
		switch (ch) {
		case 's':
			ubus_socket = optarg;
			break;
		case 'h':
			pstat_usage(argv[0]);
			exit(0);
		default:
			break;
		}
	}

	/* Logging to syslog */
	openlog("periodicstats", LOG_PID|LOG_CONS, LOG_LOCAL1);

	wait_for_dm_object();

	uloop_init();
	ctx = ubus_connect(ubus_socket);
	if (!ctx) {
		syslog(LOG_ERR, "Failed to connect to ubus\n");
		return -1;
	}

	// Initialize global periodic stats structure
	ret = pstats_global_init(&ss_list);
	if (ret) {
		syslog(LOG_ERR, "Failed to initialize global periodic stats.\n");
		goto out;
	}

	ubus_add_uloop(ctx);

	// stats collection invoked from here
	pstats_get_stats(&ss_list);

	/* Main loop of periodicstats */
	uloop_run();

 out:
	ubus_free(ctx);
	uloop_done();
	pstats_global_free(&ss_list);

	return 0;
}
